import {ApplicationState, FALLBACK_CHARSET, FALLBACK_LANGUAGE, FALLBACK_MEDIA_TYPE} from '../common';
import {Backend, BackendState, Server} from './common';
import {DataSource} from './data-source';

export interface BackendExtender<D extends DataSource = DataSource, B extends Backend<D> = Backend<D>, BB extends B = B> {
	(state: BackendState, backend: B): BB;
}

export interface CreateBackendParams<T extends DataSource = DataSource> {
	app: ApplicationState;
	dataSource: T;
}

export const createBackend = (params: CreateBackendParams) => {
	const backendState: BackendState = {
		app: params.app,
		dataSource: params.dataSource,
		cn: {
			language: FALLBACK_LANGUAGE,
			charset: FALLBACK_CHARSET,
			mediaType: FALLBACK_MEDIA_TYPE,
		},
		showTotalItemCountOnGetCollection: false,
		showTotalItemCountOnCreateItem: false,
		throwsErrorOnDeletingNotFound: false,
		checksSerializersOnDelete: false,
	};

	return {
		showTotalItemCountOnGetCollection(b = true) {
			backendState.showTotalItemCountOnGetCollection = b;
			return this;
		},
		showTotalItemCountOnCreateItem(b = true) {
			backendState.showTotalItemCountOnCreateItem = b;
			return this;
		},
		throwsErrorOnDeletingNotFound(b = true) {
			backendState.throwsErrorOnDeletingNotFound = b;
			return this;
		},
		checksSerializersOnDelete(b = true) {
			backendState.checksSerializersOnDelete = b;
			return this;
		},
		createServer(_type: string, _options = {}): Server {
			return {
				requestDecorator() {
					return this;
				},
			} satisfies Server;
		},
		use(extender) {
			return extender(backendState, this);
		},
	} satisfies Backend;
};
